from typing import Union
from pewn.classes import Option, NotSavedData
from pewn.utils import raise_error
from aiohttp import ClientSession
from aiofiles import open as aiopen
from os import makedirs, path
from asyncio import gather


async def download(url: str, option: Option = None) -> Union[str, NotSavedData]:
    """Download data from URL.

    Parameters:
        url (str): URL for fetch and download.
        option (Option): Option object. [Optional]

    Returns:
        str: Saved path.
        NotSavedData: NotSavedData object if you don't add option parameter.
    """

    raise_error(url, "url", str)

    write_file = False
    full_path = None

    if option is not None:
        raise_error(option, "option", Option)
        write_file = True

    async with ClientSession() as session:
        async with session.get(url) as response:
            data = await response.read()

            if write_file:
                if not path.isdir(option.folder):
                    makedirs(option.folder)

                full_path = f"{option.folder}/{option.file_name}"
                async with aiopen(full_path, mode="wb") as file:
                    await file.write(data)

    return full_path or NotSavedData(data, url)


async def download_multiple(urls: tuple, options: Union[tuple, Option] = None):
    """Download multiple file.

    Parameters:
        urls (tuple): List of URL that will be downloaded.
        options (tuple, Option): List of Option or only one Option object. [Optional]

    Returns:
        list (str): Saved paths.
        list (NotSavedData): List of NotSavedData object if you don't add options parameter.
    """

    raise_error(urls, "urls", tuple)

    results = ()

    if options is not None:
        raise_error(options, "option", (tuple, Option))

    if isinstance(options, tuple):
        results = await gather(*[
            download(url, opt) for url, opt in zip(urls, options)
        ])
    elif isinstance(options, Option):
        def change_file_name(option: Option, number: int):
            splitted_name = option.file_name.split('.')

            real_file_name = splitted_name[-2]
            real_file_name += f"_{number}"

            splitted_name[-2] = real_file_name

            return ".".join(splitted_name)

        results = await gather(*[
            download(url, opt) for url, opt in zip(urls, [Option(file_name=change_file_name(options, i + 1), folder=options.folder) for i, _ in enumerate(urls)])
        ])
    else:
        results = await gather(*[
            download(url) for url in urls
        ])

    return results



from dataclasses import dataclass
from pewn.utils import raise_error
from typing import Union
from aiofiles import open as aiopen
from os import makedirs, path


@dataclass
class Option:
    """Option class.

    Init:
        file_name (str): File name.
        folder (str, list): Folder list or path-like string.
            Example: ["images", "random"] is ./images/random.
            Also you can use like this:
                "./images/random"

    Attributes:
        file_name (str): File name.
        folder (str): Converted folder.
    """

    def __init__(self, file_name: str, folder: Union[str, tuple] = "./") -> None:
        raise_error(file_name, "file_name", str)
        raise_error(folder, "folder", (tuple, str))

        self.file_name: str = file_name
        self.folder: str = f"./{'/'.join(folder)}" if isinstance(folder,
                                                                 tuple) else folder

        return None


@dataclass
class NotSavedData:
    """Not saved data class. If you don't add option when you use download function, You will get this class.

    Init:
        data (bytes): Data.
        url (str): URL for data.

    Attributes:
        data (bytes): Data.
        url (str): URL for data.
        size (int): Data size in byte.

    Public Functions:
        <NotSavedData>.write(options: Option) -> str
    """

    def __init__(self, data: bytes, url: str) -> None:
        raise_error(data, "data", bytes)
        raise_error(url, "url", str)

        self.data: bytes = data
        self.url: str = url
        self.size: int = data.__sizeof__()

        return None

    async def write(self, option: Option) -> str:
        """Write data to file.

        Parameters:
            option (Option): Option object.

        Returns:
            str: Saved path.
        """

        raise_error(option, "option", Option)

        if not path.isdir(option.folder):
            makedirs(option.folder)

        full_path = f"{option.folder}/{option.file_name}"
        async with aiopen(full_path, mode="wb") as file:
            await file.write(self.data)

        return full_path



from setuptools import setup, find_packages

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
]

setup(
    name='pewn',
    version='0.1.0',
    description='📦 Another Python library for downloading files from URL.',
    long_description_content_type="text/markdown",
    long_description=open('README.md').read(),
    url='https://github.com/5elenay/pewn/',
    author='5elenay',
    author_email='',
    license='MIT',
    project_urls={
        "Bug Tracker": "https://github.com/5elenay/pewn/issues",
    },
    classifiers=classifiers,
    keywords=["python", "download", "pewn", "async", "download-file"],
    packages=find_packages(),
    python_requires='>=3.6.0',
    install_requires=["aiohttp", "aiofiles"]
)
